package com.gci.cdnev.smallprogram.modules.event.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.exception.GciBootException;
import com.gci.cdnev.smallprogram.modules.event.entity.EventInfo;
import com.gci.cdnev.smallprogram.modules.event.entity.EventProgress;
import com.gci.cdnev.smallprogram.modules.event.mapper.EventInfoMapper;
import com.gci.cdnev.smallprogram.modules.event.query.EventInfoQuery;
import com.gci.cdnev.smallprogram.modules.event.service.IEventInfoService;
import com.gci.cdnev.smallprogram.modules.event.service.IEventProgressService;
import com.gci.cdnev.smallprogram.modules.wx.entity.JwtUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 事故信息基础表 服务实现类
 * </p>
 *
 * @author 2464
 * @since 2023-09-13
 */
@Service
public class EventInfoServiceImpl extends ServiceImpl<EventInfoMapper, EventInfo> implements IEventInfoService {

    @Autowired
    private EventInfoMapper eventInfoMapper;
    @Autowired
    private IEventProgressService eventProgressService;

    @Override
    public IPage<EventInfo> pageBuild(Page<EventInfo> page, EventInfoQuery query, JwtUser user) {
        List<Date> repotTime = query.getReportTime();
        List<Date> handleTime = query.getHandleTime();
        if (ObjectUtil.isNotEmpty(repotTime)) {
            if (repotTime.size() != 2) {
                throw new GciBootException("事件上报时间必须包含起止时间！");
            }
        }
        if (ObjectUtil.isNotEmpty(handleTime)) {
            if (handleTime.size() != 2) {
                throw new GciBootException("处理时间必须包含起止时间！");
            }
        }
        QueryWrapper<EventInfo> queryWrapper = new QueryWrapper<>();

        //司机只查询自己上报的
        if (user.isDriver()) {
            queryWrapper.eq("tei.create_by", user.getUserName());
        }
        //项目方查询自己上报的或者项目内的。若传了项目。只查该项目的
        if (user.isProjectAdmin()) {
            if (StrUtil.isNotBlank(query.getProjectId())) {
                queryWrapper.eq("tei.project_id", query.getProjectId());
            } else {
                queryWrapper.and(wrapper -> {
                    wrapper.eq("tei.create_by", user.getUserName())
                            .or().in("tei.project_id", user.getProjectIds());
                });
            }
        }
        //维保部查询所有的，若传了项目，只查该项目的
        if (user.isMaintenance()) {
            if (StrUtil.isNotBlank(query.getProjectId())) {
                queryWrapper.eq("tei.project_id", query.getProjectId());
            }
        }
        if(ObjectUtil.isNotEmpty(repotTime)){
            queryWrapper.between("tei.report_time", repotTime.get(0), repotTime.get(1));
        }
        if(ObjectUtil.isNotEmpty(handleTime)){
            queryWrapper.between("tep.handle_time", handleTime.get(0), handleTime.get(1));
        }
        queryWrapper.like(StrUtil.isNotBlank(query.getEventTheme()), "tei.event_theme", query.getEventTheme());
        queryWrapper.orderByAsc("tei.is_handle");
        queryWrapper.orderByAsc("tei.report_time");
        IPage<EventInfo> pageBuild = eventInfoMapper.pagebuild(page, queryWrapper);
        return pageBuild;
    }

    @Override
    public void saveItem(EventInfo eventInfo) {
        save(eventInfo);
        EventProgress accidentProgress = eventInfo.getEventProgress();
        if (BeanUtil.isNotEmpty(accidentProgress)) {
            accidentProgress.setEventId(eventInfo.getId());
            eventProgressService.save(accidentProgress);
        }
    }

    @Override
    public void updateItem(EventInfo eventInfo,JwtUser user) {
        updateById(eventInfo);
        EventProgress eventProgress = eventInfo.getEventProgress();
        if (BeanUtil.isNotEmpty(eventProgress)) {
            eventProgress.setEventId(eventInfo.getId());
            eventProgress.setHandleBy(user.getUserName());
            if (StrUtil.isNotBlank(eventProgress.getId())) {
                eventProgressService.updateById(eventProgress);
            } else {
                eventProgressService.save(eventProgress);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handle(EventProgress progress, JwtUser user) {
        EventInfo info = this.getEventInfoById(progress.getEventId());
        if(Objects.isNull(info)){
            throw new GciBootException("找不到对应信息");
        }
        if(Objects.nonNull(info.getEventProgress())){
            throw new GciBootException("已有处理信息，不能重复处理");
        }
        info.setIsHandle(1);
        info.setUpdateBy(user.getUserName());
        info.setUpdateTime(new Date());
        updateById(info);
        if (BeanUtil.isNotEmpty(progress)) {
            progress.setHandleBy(user.getUserName());
            if (StrUtil.isNotBlank(progress.getId())) {
                eventProgressService.updateById(progress);
            } else {
                eventProgressService.save(progress);
            }
        }
    }

    @Override
    public void removeItem(String id) {
        removeById(id);
        eventProgressService.remove(new LambdaQueryWrapper<EventProgress>().eq(EventProgress::getEventId, id));
    }

    @Override
    public IPage<EventInfo> todoList(Page<EventInfo> page) {
        return eventInfoMapper.todoList(page);
    }

    @Override
    public int countTodoList() {
        return eventInfoMapper.countTodoList();
    }

    @Override
    public EventInfo getEventInfoById(String id) {
        EventInfo info = eventInfoMapper.getEventInfoById(id);
        return info;
    }

}
