package com.gci.cdnev.smallprogram.modules.system.service.impl;

import java.util.*;

import com.gci.cdnev.smallprogram.modules.system.entity.SysPermission;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gci.cdnev.smallprogram.common.vo.Result;
import com.gci.cdnev.smallprogram.constant.CommonConstant;
import com.gci.cdnev.smallprogram.modules.projects.service.IProjectsService;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUser;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUserDepart;
import com.gci.cdnev.smallprogram.modules.system.entity.SysUserRole;
import com.gci.cdnev.smallprogram.modules.system.mapper.SysUserMapper;
import com.gci.cdnev.smallprogram.modules.system.service.ISysPermissionService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysRoleService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserDepartService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserRoleService;
import com.gci.cdnev.smallprogram.modules.system.service.ISysUserService;
import com.gci.cdnev.smallprogram.modules.system.util.PasswordUtil;
import com.gci.cdnev.smallprogram.modules.system.util.QsEncodeUtil;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author chenzhiwei
 * @since 2023-08-17
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Autowired
    private ISysUserRoleService sysUserRoleService;

    @Autowired
    private ISysUserDepartService sysUserDepartService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private ISysPermissionService sysPermissionService;
    
    @Autowired
    private IProjectsService projectsService;

    @Override
    public SysUser getByUsername(String username){
    	SysUser user = lambdaQuery().eq(SysUser::getUsername, username).one();
    	initUser(user);
    	return user;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addUser(SysUser user, List<String> roleIds, List<String> departIds){
        save(user);
        addUserRoleAndDepart(user.getId(), roleIds, departIds);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void editUser(SysUser user, List<String> roleIds, List<String> departIds){
        updateById(user);
        sysUserRoleService.lambdaUpdate().eq(SysUserRole::getUserId, user.getId()).remove();
        sysUserDepartService.lambdaUpdate().eq(SysUserDepart::getUserId, user.getId()).remove();
        addUserRoleAndDepart(user.getId(), roleIds, departIds);
    }

    private void addUserRoleAndDepart(String userId, List<String> roleIds, List<String> departIds){
        if(roleIds != null && roleIds.size() > 0){
            List<SysUserRole> roleList = new ArrayList<>();
            for(String id:roleIds){
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(userId);
                sysUserRole.setRoleId(id);
                roleList.add(sysUserRole);
            }
            sysUserRoleService.saveBatch(roleList);
        }
        if(departIds != null && departIds.size()>0){
            List<SysUserDepart> depList = new ArrayList<>();
            for(String id:departIds){
                SysUserDepart sysUserDepart = new SysUserDepart();
                sysUserDepart.setUserId(userId);
                sysUserDepart.setDepId(id);
                depList.add(sysUserDepart);
            }
            sysUserDepartService.saveBatch(depList);
        }
    }


    @Override
    public String processPhone(SysUser user) {
        // 优先取phone字段
        if (StringUtils.isNotBlank(user.getPhone())) {
            return user.getPhone();
        }
        // 其次取解密后字段
        if (StringUtils.isNotBlank(user.getEncryptPhone()) && StringUtils.isNotBlank(user.getSecretPhone())) {
            try {
                return QsEncodeUtil.aesDecrypt(user.getEncryptPhone(), user.getSecretPhone());
            } catch (Exception e) {
                log.error("SysUser手机号解密失败,", e);
                return null;
            }
        }
        return null;
    }

    @Override
    public Result<?> resetPassword(String username, String oldPassword, String newPassword, String confirmPassword) {
        SysUser user = getByUsername(username);
        String passwordEncode = PasswordUtil.encrypt(username, oldPassword, user.getSalt());
        if (!user.getPassword().equals(passwordEncode)) {
            return Result.error("旧密码输入错误!");
        }
        if (StringUtils.isBlank(newPassword)) {
            return Result.error("新密码不允许为空!");
        }
        if (!newPassword.equals(confirmPassword)) {
            return Result.error("两次输入密码不一致!");
        }
        String password = PasswordUtil.encrypt(username, newPassword, user.getSalt());

        lambdaUpdate().eq(SysUser::getId, user.getId())
                .set(SysUser::getPassword, password)
                .set(SysUser::getPasswordUpdateTime, new Date())
                .update();
        return Result.ok("密码重置成功!");
    }
    @Override
    public Set<String> getUserRolesSet(String username){
        SysUser user = getByUsername(username);
        if(user != null) {
            return sysRoleService.getRoleCodeSetByUserId(user.getId());
        }
        return new HashSet<>();
    }

    @Override
    public Set<String> getUserPermissionsSet(String username) {
        Set<String> roleIds = getUserRolesSet(username);
        if(roleIds.size()>0) {
            return sysPermissionService.getPermCodeSetByroleIds(roleIds);
        }
        return new HashSet<>();
    }
    
    @Override
    public SysUser getUserById(String id) {
    	SysUser user = getById(id);
    	initUser(user);
    	return user;
    }
    
    public void initUser(SysUser user) {
    	if(user==null) {
    		return;
    	}
    	//TODO 增加其他类型查询
    	projectsService.initUserPower(user);
    }
    
    @Override
    public SysUser getByOpenid(String openid) {
        List<SysUser> list = lambdaQuery().eq(SysUser::getOpenid, openid).orderByDesc(SysUser::getCreateTime).list();
        SysUser user = list.size()>0?list.get(0):null;
    	initUser(user);
        return user;
    }

    /**
     * 根据角色编码获取用户
     */
    @Override
    public List<SysUser> getByRoleCode(String roleCode){
    	if(StringUtils.isBlank(roleCode)) {
    		roleCode = CommonConstant.MAINTENANCE_ADMIN_ROLE_CODE;
    	}
    	String sql = "select 1 from sys_role r join sys_user_role ur on r.id = ur.role_id where sys_user.id = ur.user_id and r.role_code='"+roleCode+"'";
        return lambdaQuery().exists(sql).list();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteBatch(List<String> userIds){
        removeByIds(userIds);
        sysUserRoleService.lambdaUpdate().in(SysUserRole::getUserId, userIds).remove();
        sysUserDepartService.lambdaUpdate().in(SysUserDepart::getUserId, userIds).remove();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(String userId){
        removeById(userId);
        sysUserRoleService.lambdaUpdate().eq(SysUserRole::getUserId, userId).remove();
        sysUserDepartService.lambdaUpdate().eq(SysUserDepart::getUserId, userId).remove();
    }

    /**
     *  获取权限数组
     * @param array
     * @param metaList
     */
    public void getAuthArray(List<Map<String, Object>> array, List<SysPermission> metaList) {
        for (SysPermission permission : metaList) {
            if(permission.getMenuType()==null) {
                continue;
            }
            Map<String, Object> map = null;
            if(permission.getMenuType().equals(CommonConstant.MENU_TYPE_BUTTON) && CommonConstant.STATUS_1.equals(permission.getStatus())) {
                map = new LinkedHashMap<>();
                map.put("action", permission.getPerms());
                map.put("type", permission.getPermsType());
                map.put("describe", permission.getName());
                array.add(map);
            }
        }
    }

}
